/*
**	Command & Conquer Renegade(tm)
**	Copyright 2025 Electronic Arts Inc.
**
**	This program is free software: you can redistribute it and/or modify
**	it under the terms of the GNU General Public License as published by
**	the Free Software Foundation, either version 3 of the License, or
**	(at your option) any later version.
**
**	This program is distributed in the hope that it will be useful,
**	but WITHOUT ANY WARRANTY; without even the implied warranty of
**	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**	GNU General Public License for more details.
**
**	You should have received a copy of the GNU General Public License
**	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/***********************************************************************************************
 ***              C O N F I D E N T I A L  ---  W E S T W O O D  S T U D I O S               ***
 ***********************************************************************************************
 *                                                                                             *
 *                 Project Name : WWPhys                                                       *
 *                                                                                             *
 *                     $Archive:: /Commando/Code/wwphys/bin_axes.cpp                          $*
 *                                                                                             *
 *                       Author:: Greg Hjelstrom                                               *
 *                                                                                             *
 *                     $Modtime:: 7/29/99 11:27a                                              $*
 *                                                                                             *
 *                    $Revision:: 2                                                           $*
 *                                                                                             *
 *---------------------------------------------------------------------------------------------*
 * Functions:                                                                                  *
 * - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */


#include "bin_axes.h"

#ifdef WWDEBUG
const unsigned char _AxesBinary[9224] = 
{
	0x00,0x00,0x00,0x00,0x00,0x24,0x00,0x00,0x1F,0x00,0x00,0x00,0x74,0x00,0x00,0x00,
	0x01,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x41,0x58,0x45,0x53,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x90,0x00,0x00,0x00,0x86,0x00,0x00,0x00,
	0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x03,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x78,0xBE,0x5B,0xBE,
	0xEE,0xB6,0x4F,0xBE,0x71,0xB9,0x53,0xBE,0xBE,0xFE,0x7D,0x3F,0x50,0x80,0x80,0x3F,
	0x00,0x00,0x80,0x3F,0xD0,0x9B,0x34,0x3E,0x53,0x2F,0xDD,0x3D,0xE0,0xFA,0x84,0x3E,
	0xBD,0x14,0x74,0x3F,0x02,0x00,0x00,0x00,0x48,0x06,0x00,0x00,0x6A,0x50,0x00,0xBC,
	0x6A,0x50,0x80,0x3B,0x71,0xB9,0x53,0x3E,0x71,0x50,0x00,0xBC,0xB8,0xB8,0x19,0x3E,
	0x35,0xB6,0x15,0x3E,0x7F,0xC3,0xE3,0xBD,0x77,0xBE,0xDB,0x3D,0x35,0xB6,0x15,0x3E,
	0x3C,0xBB,0x1D,0xBE,0x4F,0x50,0x80,0x3B,0x35,0xB6,0x15,0x3E,0x7D,0xC3,0xE3,0xBD,
	0x6C,0xB4,0xCB,0xBD,0x35,0xB6,0x15,0x3E,0x55,0x50,0x00,0xBC,0xB2,0xB3,0x11,0xBE,
	0x35,0xB6,0x15,0x3E,0x66,0xAF,0xC3,0x3D,0x68,0xB4,0xCB,0xBD,0x35,0xB6,0x15,0x3E,
	0x2E,0xB1,0x0D,0x3E,0xA1,0x50,0x80,0x3B,0x35,0xB6,0x15,0x3E,0x61,0xAF,0xC3,0x3D,
	0x7B,0xBE,0xDB,0x3D,0x35,0xB6,0x15,0x3E,0x74,0x50,0x00,0xBC,0xF4,0xBB,0x57,0x3E,
	0xEC,0x44,0x1B,0xB2,0x3D,0xBB,0x1D,0xBE,0xB7,0xB8,0x19,0x3E,0xEC,0x44,0x1B,0xB2,
	0x78,0xBE,0x5B,0xBE,0x43,0x50,0x80,0x3B,0xEC,0x44,0x1B,0xB2,0x3B,0xBB,0x1D,0xBE,
	0xB3,0xB3,0x11,0xBE,0xEC,0x44,0x1B,0xB2,0x4D,0x50,0x00,0xBC,0xEE,0xB6,0x4F,0xBE,
	0xEC,0x44,0x1B,0xB2,0x2F,0xB1,0x0D,0x3E,0xB0,0xB3,0x11,0xBE,0xEC,0x44,0x1B,0xB2,
	0x6A,0xB4,0x4B,0x3E,0xB8,0x50,0x80,0x3B,0xEC,0x44,0x1B,0xB2,0x2C,0xB1,0x0D,0x3E,
	0xBA,0xB8,0x19,0x3E,0xEC,0x44,0x1B,0xB2,0x71,0x50,0x00,0xBC,0xB7,0xB8,0x19,0x3E,
	0x36,0xB6,0x15,0xBE,0x7E,0xC3,0xE3,0xBD,0x76,0xBE,0xDB,0x3D,0x36,0xB6,0x15,0xBE,
	0x3B,0xBB,0x1D,0xBE,0x4F,0x50,0x80,0x3B,0x36,0xB6,0x15,0xBE,0x7B,0xC3,0xE3,0xBD,
	0x6A,0xB4,0xCB,0xBD,0x36,0xB6,0x15,0xBE,0x55,0x50,0x00,0xBC,0xB1,0xB3,0x11,0xBE,
	0x36,0xB6,0x15,0xBE,0x65,0xAF,0xC3,0x3D,0x67,0xB4,0xCB,0xBD,0x36,0xB6,0x15,0xBE,
	0x2D,0xB1,0x0D,0x3E,0xA1,0x50,0x80,0x3B,0x36,0xB6,0x15,0xBE,0x60,0xAF,0xC3,0x3D,
	0x79,0xBE,0xDB,0x3D,0x36,0xB6,0x15,0xBE,0x6A,0x50,0x00,0xBC,0x6A,0x50,0x80,0x3B,
	0x71,0xB9,0x53,0xBE,0x6A,0x50,0x00,0xBC,0x6A,0x50,0x80,0x3B,0x00,0x00,0x00,0x00,
	0x68,0x50,0x00,0xBC,0x5F,0x50,0x80,0x3B,0x8F,0xC2,0x75,0x3D,0x67,0x50,0x00,0xBC,
	0x4E,0xE6,0x82,0x3D,0x8F,0xC2,0xF5,0x30,0x6D,0x50,0x00,0xBC,0x82,0xB8,0x65,0xBD,
	0x60,0xAB,0xF1,0xB1,0x6C,0x50,0x00,0xBC,0x81,0x50,0x80,0x3B,0x8F,0xC2,0x75,0xBD,
	0x67,0x50,0x00,0xBC,0x4E,0xE6,0x82,0x3D,0x8F,0xC2,0xF5,0x30,0xBE,0xFE,0x3D,0x3F,
	0x2F,0x50,0x80,0x3B,0x89,0xC2,0x75,0x3D,0xBE,0xFE,0x3D,0x3F,0x4B,0xE6,0x82,0x3D,
	0xD7,0xA3,0xB0,0xB2,0x68,0x50,0x00,0xBC,0x5F,0x50,0x80,0x3B,0x8F,0xC2,0x75,0x3D,
	0xBE,0xFE,0x3D,0x3F,0x88,0xB8,0x65,0xBD,0xD8,0x6A,0xFC,0xB2,0x6D,0x50,0x00,0xBC,
	0x82,0xB8,0x65,0xBD,0x60,0xAB,0xF1,0xB1,0xBE,0xFE,0x3D,0x3F,0x51,0x50,0x80,0x3B,
	0x95,0xC2,0x75,0xBD,0x6C,0x50,0x00,0xBC,0x81,0x50,0x80,0x3B,0x8F,0xC2,0x75,0xBD,
	0xBE,0xFE,0x3D,0x3F,0x3A,0x50,0x80,0x3B,0x00,0x00,0xC0,0xB2,0xBE,0xFE,0x3D,0x3F,
	0x4B,0xE6,0x82,0x3D,0xD7,0xA3,0xB0,0xB2,0xBE,0xFE,0x3D,0x3F,0x2F,0x50,0x80,0x3B,
	0x89,0xC2,0x75,0x3D,0xBE,0xFE,0x3D,0x3F,0x88,0xB8,0x65,0xBD,0xD8,0x6A,0xFC,0xB2,
	0xBE,0xFE,0x3D,0x3F,0x51,0x50,0x80,0x3B,0x95,0xC2,0x75,0xBD,0xBE,0xFE,0x3D,0x3F,
	0x87,0x78,0x40,0x3C,0x99,0x50,0x80,0xBB,0xBE,0xFE,0x3D,0x3F,0x78,0x78,0x40,0x3C,
	0x86,0xD4,0x1F,0x3E,0xBE,0xFE,0x3D,0x3F,0x93,0xDE,0x2F,0x3E,0x8F,0x50,0x80,0xBB,
	0xBE,0xFE,0x3D,0x3F,0x81,0xCF,0x17,0xBE,0xC2,0x50,0x80,0xBB,0xBE,0xFE,0x3D,0x3F,
	0xA6,0x78,0x40,0x3C,0x8E,0xD9,0x27,0xBE,0xBE,0xFE,0x3D,0x3F,0x93,0xDE,0x2F,0x3E,
	0x8F,0x50,0x80,0xBB,0xBE,0xFE,0x7D,0x3F,0x7F,0x78,0x40,0x3C,0xA9,0x50,0x80,0xBB,
	0xBE,0xFE,0x7D,0x3F,0x7F,0x78,0x40,0x3C,0xA9,0x50,0x80,0xBB,0xBE,0xFE,0x3D,0x3F,
	0x78,0x78,0x40,0x3C,0x86,0xD4,0x1F,0x3E,0xBE,0xFE,0x7D,0x3F,0x7F,0x78,0x40,0x3C,
	0xA9,0x50,0x80,0xBB,0xBE,0xFE,0x3D,0x3F,0x81,0xCF,0x17,0xBE,0xC2,0x50,0x80,0xBB,
	0xBE,0xFE,0x7D,0x3F,0x7F,0x78,0x40,0x3C,0xA9,0x50,0x80,0xBB,0xBE,0xFE,0x3D,0x3F,
	0xA6,0x78,0x40,0x3C,0x8E,0xD9,0x27,0xBE,0xBE,0xFE,0x7D,0x3F,0x7F,0x78,0x40,0x3C,
	0xA9,0x50,0x80,0xBB,0xBE,0xFE,0x7D,0x3F,0x7F,0x78,0x40,0x3C,0xA9,0x50,0x80,0xBB,
	0xBE,0xFE,0x7D,0x3F,0x7F,0x78,0x40,0x3C,0xA9,0x50,0x80,0xBB,0xBE,0xFE,0x7D,0x3F,
	0x7F,0x78,0x40,0x3C,0xA9,0x50,0x80,0xBB,0xBE,0xFE,0x7D,0x3F,0x7F,0x78,0x40,0x3C,
	0xA9,0x50,0x80,0xBB,0x6A,0x50,0x00,0xBC,0x6A,0x50,0x80,0x3B,0x00,0x00,0x00,0x00,
	0x65,0x50,0x00,0xBC,0x82,0xB8,0x65,0xBD,0x50,0xC7,0xAD,0x31,0x55,0xEB,0x8A,0xBD,
	0x66,0x50,0x80,0x3B,0x8F,0xC2,0xF5,0x30,0x74,0xAE,0x55,0x3D,0x79,0x50,0x80,0x3B,
	0x93,0xC2,0xF5,0xB0,0x74,0x50,0x00,0xBC,0x4E,0xE6,0x82,0x3D,0x50,0xC7,0xAD,0xB1,
	0x55,0xEB,0x8A,0xBD,0x66,0x50,0x80,0x3B,0x8F,0xC2,0xF5,0x30,0x4D,0x50,0x00,0xBC,
	0x71,0xB8,0x65,0xBD,0x00,0x00,0x40,0x3F,0x52,0xEB,0x8A,0xBD,0xEE,0x50,0x80,0x3B,
	0x00,0x00,0x40,0x3F,0x65,0x50,0x00,0xBC,0x82,0xB8,0x65,0xBD,0x50,0xC7,0xAD,0x31,
	0x7A,0xAE,0x55,0x3D,0x01,0x51,0x80,0x3B,0x00,0x00,0x40,0x3F,0x74,0xAE,0x55,0x3D,
	0x79,0x50,0x80,0x3B,0x93,0xC2,0xF5,0xB0,0x5C,0x50,0x00,0xBC,0x57,0xE6,0x82,0x3D,
	0x00,0x00,0x40,0x3F,0x74,0x50,0x00,0xBC,0x4E,0xE6,0x82,0x3D,0x50,0xC7,0xAD,0xB1,
	0x52,0x50,0x00,0xBC,0xF2,0x50,0x80,0x3B,0x00,0x00,0x40,0x3F,0x52,0xEB,0x8A,0xBD,
	0xEE,0x50,0x80,0x3B,0x00,0x00,0x40,0x3F,0x4D,0x50,0x00,0xBC,0x71,0xB8,0x65,0xBD,
	0x00,0x00,0x40,0x3F,0x7A,0xAE,0x55,0x3D,0x01,0x51,0x80,0x3B,0x00,0x00,0x40,0x3F,
	0x5C,0x50,0x00,0xBC,0x57,0xE6,0x82,0x3D,0x00,0x00,0x40,0x3F,0x5E,0x50,0x80,0xBC,
	0xAE,0x50,0x00,0x3C,0x00,0x00,0x40,0x3F,0x58,0x50,0x80,0xBC,0x00,0xD2,0x1B,0xBE,
	0x00,0x00,0x40,0x3F,0x17,0xE1,0x33,0xBE,0xA8,0x50,0x00,0x3C,0x00,0x00,0x40,0x3F,
	0xFD,0xCC,0x13,0x3E,0xC2,0x50,0x00,0x3C,0x00,0x00,0x40,0x3F,0x6C,0x50,0x80,0xBC,
	0x14,0xDC,0x2B,0x3E,0x00,0x00,0x40,0x3F,0x17,0xE1,0x33,0xBE,0xA8,0x50,0x00,0x3C,
	0x00,0x00,0x40,0x3F,0x5A,0x50,0x80,0xBC,0xC4,0x50,0x00,0x3C,0x00,0x00,0x80,0x3F,
	0x5A,0x50,0x80,0xBC,0xC4,0x50,0x00,0x3C,0x00,0x00,0x80,0x3F,0x58,0x50,0x80,0xBC,
	0x00,0xD2,0x1B,0xBE,0x00,0x00,0x40,0x3F,0x5A,0x50,0x80,0xBC,0xC4,0x50,0x00,0x3C,
	0x00,0x00,0x80,0x3F,0xFD,0xCC,0x13,0x3E,0xC2,0x50,0x00,0x3C,0x00,0x00,0x40,0x3F,
	0x5A,0x50,0x80,0xBC,0xC4,0x50,0x00,0x3C,0x00,0x00,0x80,0x3F,0x6C,0x50,0x80,0xBC,
	0x14,0xDC,0x2B,0x3E,0x00,0x00,0x40,0x3F,0x5A,0x50,0x80,0xBC,0xC4,0x50,0x00,0x3C,
	0x00,0x00,0x80,0x3F,0x5A,0x50,0x80,0xBC,0xC4,0x50,0x00,0x3C,0x00,0x00,0x80,0x3F,
	0x5A,0x50,0x80,0xBC,0xC4,0x50,0x00,0x3C,0x00,0x00,0x80,0x3F,0x5A,0x50,0x80,0xBC,
	0xC4,0x50,0x00,0x3C,0x00,0x00,0x80,0x3F,0x5A,0x50,0x80,0xBC,0xC4,0x50,0x00,0x3C,
	0x00,0x00,0x80,0x3F,0x6A,0x50,0x00,0xBC,0x6A,0x50,0x80,0x3B,0x00,0x00,0x00,0x00,
	0x65,0x50,0x00,0xBC,0x75,0x50,0x80,0x3B,0x8F,0xC2,0x75,0x3D,0x54,0xEB,0x8A,0xBD,
	0x6E,0x50,0x80,0x3B,0x8F,0xC2,0xF5,0x30,0x74,0xAE,0x55,0x3D,0x66,0x50,0x80,0x3B,
	0x60,0xAB,0xF1,0xB1,0x74,0x50,0x00,0xBC,0x5F,0x50,0x80,0x3B,0x8F,0xC2,0x75,0xBD,
	0x54,0xEB,0x8A,0xBD,0x6E,0x50,0x80,0x3B,0x8F,0xC2,0xF5,0x30,0x4D,0x50,0x00,0xBC,
	0xA1,0x00,0x41,0x3F,0x8F,0xC2,0x75,0x3D,0x51,0xEB,0x8A,0xBD,0xA1,0x00,0x41,0x3F,
	0x89,0xC2,0xF5,0x30,0x65,0x50,0x00,0xBC,0x75,0x50,0x80,0x3B,0x8F,0xC2,0x75,0x3D,
	0x7A,0xAE,0x55,0x3D,0xA1,0x00,0x41,0x3F,0x61,0xAB,0xF1,0xB1,0x74,0xAE,0x55,0x3D,
	0x66,0x50,0x80,0x3B,0x60,0xAB,0xF1,0xB1,0x5C,0x50,0x00,0xBC,0xA1,0x00,0x41,0x3F,
	0x8F,0xC2,0x75,0xBD,0x74,0x50,0x00,0xBC,0x5F,0x50,0x80,0x3B,0x8F,0xC2,0x75,0xBD,
	0x52,0x50,0x00,0xBC,0xA1,0x00,0x41,0x3F,0x00,0x00,0x40,0xA6,0x51,0xEB,0x8A,0xBD,
	0xA1,0x00,0x41,0x3F,0x89,0xC2,0xF5,0x30,0x4D,0x50,0x00,0xBC,0xA1,0x00,0x41,0x3F,
	0x8F,0xC2,0x75,0x3D,0x7A,0xAE,0x55,0x3D,0xA1,0x00,0x41,0x3F,0x61,0xAB,0xF1,0xB1,
	0x5C,0x50,0x00,0xBC,0xA1,0x00,0x41,0x3F,0x8F,0xC2,0x75,0xBD,0x5E,0x50,0x80,0xBC,
	0xA1,0x00,0x41,0x3F,0x69,0x50,0x80,0xBB,0x58,0x50,0x80,0xBC,0xA1,0x00,0x41,0x3F,
	0x87,0xD4,0x1F,0x3E,0x16,0xE1,0x33,0xBE,0xA1,0x00,0x41,0x3F,0x5F,0x50,0x80,0xBB,
	0xFD,0xCC,0x13,0x3E,0xA1,0x00,0x41,0x3F,0x92,0x50,0x80,0xBB,0x6C,0x50,0x80,0xBC,
	0xA1,0x00,0x41,0x3F,0x8D,0xD9,0x27,0xBE,0x16,0xE1,0x33,0xBE,0xA1,0x00,0x41,0x3F,
	0x5F,0x50,0x80,0xBB,0x5A,0x50,0x80,0xBC,0x50,0x80,0x80,0x3F,0x69,0x50,0x80,0xBB,
	0x5A,0x50,0x80,0xBC,0x50,0x80,0x80,0x3F,0x69,0x50,0x80,0xBB,0x58,0x50,0x80,0xBC,
	0xA1,0x00,0x41,0x3F,0x87,0xD4,0x1F,0x3E,0x5A,0x50,0x80,0xBC,0x50,0x80,0x80,0x3F,
	0x69,0x50,0x80,0xBB,0xFD,0xCC,0x13,0x3E,0xA1,0x00,0x41,0x3F,0x92,0x50,0x80,0xBB,
	0x5A,0x50,0x80,0xBC,0x50,0x80,0x80,0x3F,0x69,0x50,0x80,0xBB,0x6C,0x50,0x80,0xBC,
	0xA1,0x00,0x41,0x3F,0x8D,0xD9,0x27,0xBE,0x5A,0x50,0x80,0xBC,0x50,0x80,0x80,0x3F,
	0x69,0x50,0x80,0xBB,0x5A,0x50,0x80,0xBC,0x50,0x80,0x80,0x3F,0x69,0x50,0x80,0xBB,
	0x5A,0x50,0x80,0xBC,0x50,0x80,0x80,0x3F,0x69,0x50,0x80,0xBB,0x5A,0x50,0x80,0xBC,
	0x50,0x80,0x80,0x3F,0x69,0x50,0x80,0xBB,0x5A,0x50,0x80,0xBC,0x50,0x80,0x80,0x3F,
	0x69,0x50,0x80,0xBB,0x03,0x00,0x00,0x00,0x48,0x06,0x00,0x00,0x00,0x00,0x00,0x00,
	0xA6,0x46,0x0C,0x32,0x00,0x00,0x80,0x3F,0x96,0xE2,0xA4,0xBD,0x16,0xDE,0x40,0x3F,
	0xA3,0x12,0x27,0x3F,0xAC,0xF3,0x16,0xBF,0xA9,0x9B,0xF3,0x3E,0xA4,0x12,0x27,0x3F,
	0x15,0xDE,0x40,0xBF,0x91,0xE2,0xA4,0xBD,0xA4,0x12,0x27,0x3F,0xA6,0x9B,0xF3,0xBE,
	0xAC,0xF3,0x16,0xBF,0xA3,0x12,0x27,0x3F,0x9B,0xE2,0xA4,0x3D,0x15,0xDE,0x40,0xBF,
	0xA3,0x12,0x27,0x3F,0xAD,0xF3,0x16,0x3F,0xA7,0x9B,0xF3,0xBE,0xA4,0x12,0x27,0x3F,
	0x16,0xDE,0x40,0x3F,0x9B,0xE2,0xA4,0x3D,0xA3,0x12,0x27,0x3F,0xA7,0x9B,0xF3,0x3E,
	0xAD,0xF3,0x16,0x3F,0xA3,0x12,0x27,0x3F,0xE2,0xB4,0xA0,0xB3,0x00,0x00,0x80,0x3F,
	0xE2,0xB4,0xA0,0xB3,0xF5,0x04,0x35,0xBF,0xF3,0x04,0x35,0x3F,0x34,0x58,0x94,0xB3,
	0x00,0x00,0x80,0xBF,0xF0,0xCA,0xC5,0xB3,0x41,0x6E,0xB9,0xB3,0xF2,0x04,0x35,0xBF,
	0xF5,0x04,0x35,0xBF,0xE3,0xB4,0xA0,0xB3,0xE2,0xB4,0x20,0x34,0x00,0x00,0x80,0xBF,
	0x91,0x11,0x2D,0xB3,0xF5,0x04,0x35,0x3F,0xF1,0x04,0x35,0xBF,0x91,0x11,0x2D,0xB3,
	0x00,0x00,0x80,0x3F,0x39,0xE3,0x26,0x34,0x84,0xFB,0x87,0xB3,0xF2,0x04,0x35,0x3F,
	0xF4,0x04,0x35,0x3F,0x4C,0x84,0x5E,0xB3,0x83,0xE2,0xA4,0x3D,0x15,0xDE,0x40,0x3F,
	0xA6,0x12,0x27,0xBF,0xA8,0x9B,0xF3,0xBE,0xAA,0xF3,0x16,0x3F,0xA4,0x12,0x27,0xBF,
	0x15,0xDE,0x40,0xBF,0x7C,0xE2,0xA4,0x3D,0xA5,0x12,0x27,0xBF,0xAA,0xF3,0x16,0xBF,
	0xA9,0x9B,0xF3,0xBE,0xA5,0x12,0x27,0xBF,0x7C,0xE2,0xA4,0xBD,0x14,0xDE,0x40,0xBF,
	0xA5,0x12,0x27,0xBF,0xAB,0x9B,0xF3,0x3E,0xA9,0xF3,0x16,0xBF,0xA4,0x12,0x27,0xBF,
	0x15,0xDE,0x40,0x3F,0x78,0xE2,0xA4,0xBD,0xA4,0x12,0x27,0xBF,0xAA,0xF3,0x16,0x3F,
	0xA9,0x9B,0xF3,0x3E,0xA4,0x12,0x27,0xBF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x80,0xBF,0x00,0x00,0x80,0xBF,0x00,0x00,0x48,0x33,0x57,0x55,0x05,0x33,
	0x00,0x00,0x80,0xBF,0x00,0x00,0x48,0x33,0x56,0x55,0x05,0x33,0x00,0x00,0x80,0xBF,
	0x00,0x00,0x48,0x33,0x56,0x55,0x05,0x33,0x00,0x00,0x80,0xBF,0xFF,0xFF,0x47,0x33,
	0x56,0x55,0x05,0x33,0x00,0x00,0x80,0xBF,0x00,0x00,0x48,0x33,0x57,0x55,0x05,0x33,
	0x9D,0xE8,0x21,0x33,0xE8,0xDC,0x72,0x3F,0x9D,0xE8,0xA1,0x3E,0x9C,0xE8,0x21,0x33,
	0x9A,0xE8,0xA1,0x3E,0xE9,0xDC,0x72,0x3F,0x9D,0xE8,0xA1,0x32,0xE9,0xDC,0x72,0x3F,
	0x98,0xE8,0xA1,0xBE,0x9A,0xE8,0xA1,0x32,0x9E,0xE8,0xA1,0xBE,0xE8,0xDC,0x72,0x3F,
	0x9D,0xE8,0xA1,0xB2,0xE9,0xDC,0x72,0xBF,0x96,0xE8,0xA1,0x3E,0x9C,0xE8,0x21,0xB3,
	0xE8,0xDC,0x72,0xBF,0x9F,0xE8,0xA1,0xBE,0x9A,0xE8,0x21,0xB3,0x97,0xE8,0xA1,0xBE,
	0xEA,0xDC,0x72,0xBF,0x9A,0xE8,0xA1,0xB2,0x9D,0xE8,0xA1,0x3E,0xE8,0xDC,0x72,0xBF,
	0x00,0x00,0x80,0x3F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x80,0x3F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x80,0xBF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x6D,0xFF,0x09,0x3F,0x19,0x9F,0x57,0x3F,
	0x78,0x5A,0x27,0x33,0x44,0x18,0xD3,0x3E,0xF3,0xEA,0x24,0x3F,0xF5,0xEA,0x24,0x3F,
	0x43,0x18,0xD3,0x3E,0xF4,0xEA,0x24,0x3F,0xF3,0xEA,0x24,0xBF,0x6D,0xFF,0x09,0x3F,
	0x78,0x5A,0x27,0xB4,0x19,0x9F,0x57,0x3F,0x41,0x18,0xD3,0x3E,0xF6,0xEA,0x24,0xBF,
	0xF2,0xEA,0x24,0x3F,0x6B,0xFF,0x09,0x3F,0x1A,0x9F,0x57,0xBF,0xB4,0x07,0xFB,0xB3,
	0x41,0x18,0xD3,0x3E,0xF3,0xEA,0x24,0xBF,0xF6,0xEA,0x24,0xBF,0x6C,0xFF,0x09,0x3F,
	0x77,0x5A,0xA7,0x33,0x19,0x9F,0x57,0xBF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0xFE,0xFF,0xFF,0xB2,0xF4,0x04,0xB5,0xB3,0x00,0x00,0x80,0xBF,
	0xFE,0xFF,0xFF,0xB2,0xF4,0x04,0xB5,0xB3,0x00,0x00,0x80,0xBF,0xFD,0xFF,0xFF,0xB2,
	0xF4,0x04,0xB5,0xB3,0x00,0x00,0x80,0xBF,0xFF,0xFF,0xFF,0xB2,0xF4,0x04,0xB5,0xB3,
	0x00,0x00,0x80,0xBF,0xFE,0xFF,0xFF,0xB2,0xF4,0x04,0xB5,0xB3,0x00,0x00,0x80,0xBF,
	0xE8,0xDC,0x72,0xBF,0x9E,0xE8,0xA1,0xBE,0xE4,0x1D,0x6C,0x33,0x99,0xE8,0xA1,0xBE,
	0xE9,0xDC,0x72,0xBF,0x38,0x44,0xC0,0x33,0xE9,0xDC,0x72,0xBF,0x98,0xE8,0xA1,0x3E,
	0xE2,0xBA,0x2B,0x28,0x9E,0xE8,0xA1,0x3E,0xE8,0xDC,0x72,0xBF,0x10,0xCA,0x97,0x33,
	0xE9,0xDC,0x72,0x3F,0x97,0xE8,0xA1,0xBE,0x50,0xE1,0xD7,0xB0,0xE8,0xDC,0x72,0x3F,
	0x9F,0xE8,0xA1,0x3E,0xEB,0xDC,0x72,0xB3,0x98,0xE8,0xA1,0x3E,0xE9,0xDC,0x72,0x3F,
	0x3E,0x03,0xC7,0xB3,0x9C,0xE8,0xA1,0xBE,0xE8,0xDC,0x72,0x3F,0x16,0x89,0x9E,0xB3,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x80,0x3F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x80,0xBF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,0x18,0x9F,0x57,0xBF,0x00,0x00,0x00,0x00,
	0x6D,0xFF,0x09,0x3F,0xF2,0xEA,0x24,0xBF,0xF4,0xEA,0x24,0xBF,0x46,0x18,0xD3,0x3E,
	0xF4,0xEA,0x24,0xBF,0xF4,0xEA,0x24,0x3F,0x42,0x18,0xD3,0x3E,0x77,0x5A,0x27,0x34,
	0x18,0x9F,0x57,0xBF,0x6D,0xFF,0x09,0x3F,0xF6,0xEA,0x24,0x3F,0xF2,0xEA,0x24,0xBF,
	0x42,0x18,0xD3,0x3E,0x1B,0x9F,0x57,0x3F,0x77,0x5A,0x27,0x34,0x6B,0xFF,0x09,0x3F,
	0xF4,0xEA,0x24,0x3F,0xF6,0xEA,0x24,0x3F,0x3F,0x18,0xD3,0x3E,0x00,0x00,0x00,0x00,
	0x1B,0x9F,0x57,0x3F,0x6B,0xFF,0x09,0x3F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x54,0x55,0x05,0xB3,0x00,0x00,0x80,0xBF,0x56,0x55,0xB7,0x33,
	0x54,0x55,0x05,0xB3,0x00,0x00,0x80,0xBF,0x56,0x55,0xB7,0x33,0x55,0x55,0x05,0xB3,
	0x00,0x00,0x80,0xBF,0x56,0x55,0xB7,0x33,0x53,0x55,0x05,0xB3,0x00,0x00,0x80,0xBF,
	0x56,0x55,0xB7,0x33,0x54,0x55,0x05,0xB3,0x00,0x00,0x80,0xBF,0x56,0x55,0xB7,0x33,
	0xE9,0xDC,0x72,0xBF,0xE9,0xDC,0xF2,0x32,0x9C,0xE8,0xA1,0x3E,0x9A,0xE8,0xA1,0xBE,
	0x9C,0xE8,0x21,0x32,0xE9,0xDC,0x72,0x3F,0xE9,0xDC,0x72,0xBF,0xE9,0xDC,0xF2,0x32,
	0x99,0xE8,0xA1,0xBE,0x9E,0xE8,0xA1,0x3E,0x9E,0xE8,0x21,0xB2,0xE8,0xDC,0x72,0x3F,
	0xE9,0xDC,0x72,0x3F,0xE9,0xDC,0xF2,0xB2,0x97,0xE8,0xA1,0x3E,0xE8,0xDC,0x72,0x3F,
	0xE8,0xDC,0xF2,0xB2,0x9F,0xE8,0xA1,0xBE,0x97,0xE8,0xA1,0x3E,0x97,0xE8,0x21,0xB2,
	0xE9,0xDC,0x72,0xBF,0x9F,0xE8,0xA1,0xBE,0x9F,0xE8,0x21,0x32,0xE8,0xDC,0x72,0xBF,
	0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x80,0x3F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x80,0xBF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x80,0xBF,0x00,0x00,0x00,0x00,0x18,0x9F,0x57,0xBF,0x6E,0xFF,0x09,0x3F,
	0x77,0x5A,0xA7,0x33,0xF3,0xEA,0x24,0xBF,0x46,0x18,0xD3,0x3E,0xF4,0xEA,0x24,0x3F,
	0xF4,0xEA,0x24,0xBF,0x46,0x18,0xD3,0x3E,0xF2,0xEA,0x24,0xBF,0xB3,0x07,0xFB,0x33,
	0x6D,0xFF,0x09,0x3F,0x18,0x9F,0x57,0x3F,0xF6,0xEA,0x24,0x3F,0x43,0x18,0xD3,0x3E,
	0xF2,0xEA,0x24,0x3F,0x19,0x9F,0x57,0x3F,0x6D,0xFF,0x09,0x3F,0xB3,0x07,0xFB,0xB3,
	0xF3,0xEA,0x24,0x3F,0x43,0x18,0xD3,0x3E,0xF5,0xEA,0x24,0xBF,0x77,0x5A,0x27,0xB3,
	0x6D,0xFF,0x09,0x3F,0x18,0x9F,0x57,0xBF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x00,0x12,0x00,0x00,0x35,0x00,0x00,0x00,
	0x36,0x00,0x00,0x00,0x37,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x37,0x00,0x00,0x00,
	0x36,0x00,0x00,0x00,0x38,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x40,0x18,0xD3,0x3E,
	0xF2,0xEA,0x24,0xBF,0xF5,0xEA,0x24,0xBF,0xBD,0xDB,0xCE,0x3E,0x37,0x00,0x00,0x00,
	0x38,0x00,0x00,0x00,0x33,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x33,0x00,0x00,0x00,
	0x38,0x00,0x00,0x00,0x31,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x43,0x18,0xD3,0x3E,
	0xF4,0xEA,0x24,0x3F,0xF3,0xEA,0x24,0xBF,0x9D,0x9B,0xD6,0x3E,0x33,0x00,0x00,0x00,
	0x31,0x00,0x00,0x00,0x32,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x32,0x00,0x00,0x00,
	0x31,0x00,0x00,0x00,0x34,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x43,0x18,0xD3,0x3E,
	0xF2,0xEA,0x24,0x3F,0xF4,0xEA,0x24,0x3F,0x52,0x06,0xD4,0x3E,0x32,0x00,0x00,0x00,
	0x34,0x00,0x00,0x00,0x35,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x35,0x00,0x00,0x00,
	0x34,0x00,0x00,0x00,0x36,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x41,0x18,0xD3,0x3E,
	0xF6,0xEA,0x24,0xBF,0xF2,0xEA,0x24,0x3F,0x73,0x46,0xCC,0x3E,0x1C,0x00,0x00,0x00,
	0x1A,0x00,0x00,0x00,0x1B,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,
	0x00,0x00,0x48,0x33,0x56,0x55,0x05,0x33,0x6A,0x50,0x00,0x3C,0x1B,0x00,0x00,0x00,
	0x1A,0x00,0x00,0x00,0x1D,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,
	0xFF,0xFF,0x47,0x33,0x56,0x55,0x05,0x33,0x6A,0x50,0x00,0x3C,0x1D,0x00,0x00,0x00,
	0x1A,0x00,0x00,0x00,0x1E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,
	0xFF,0xFF,0x47,0x33,0x57,0x55,0x05,0x33,0x6A,0x50,0x00,0x3C,0x1E,0x00,0x00,0x00,
	0x1A,0x00,0x00,0x00,0x1C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,
	0x00,0x00,0x48,0x33,0x57,0x55,0x05,0x33,0x6A,0x50,0x00,0x3C,0x21,0x00,0x00,0x00,
	0x1F,0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF4,0x04,0x35,0x33,
	0xF3,0x04,0x35,0x3F,0xF4,0x04,0x35,0x3F,0xBA,0x1E,0x39,0x3D,0x20,0x00,0x00,0x00,
	0x1F,0x00,0x00,0x00,0x22,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF4,0x04,0x35,0x33,
	0xF3,0x04,0x35,0x3F,0xF4,0x04,0x35,0x3F,0xBA,0x1E,0x39,0x3D,0x20,0x00,0x00,0x00,
	0x22,0x00,0x00,0x00,0x23,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xDE,0x6B,0x8D,0xA7,
	0xF5,0x04,0x35,0xBF,0xF2,0x04,0x35,0x3F,0xE5,0x6F,0x22,0x3D,0x23,0x00,0x00,0x00,
	0x22,0x00,0x00,0x00,0x24,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xDE,0x6B,0x8D,0xA7,
	0xF5,0x04,0x35,0xBF,0xF3,0x04,0x35,0x3F,0xE6,0x6F,0x22,0x3D,0x23,0x00,0x00,0x00,
	0x24,0x00,0x00,0x00,0x25,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF4,0x04,0x35,0xB3,
	0xF2,0x04,0x35,0xBF,0xF6,0x04,0x35,0xBF,0xE5,0x6F,0x22,0x3D,0x25,0x00,0x00,0x00,
	0x24,0x00,0x00,0x00,0x26,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF4,0x04,0x35,0xB3,
	0xF2,0x04,0x35,0xBF,0xF5,0x04,0x35,0xBF,0xE5,0x6F,0x22,0x3D,0x25,0x00,0x00,0x00,
	0x26,0x00,0x00,0x00,0x21,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xDF,0x6B,0x8D,0x27,
	0xF4,0x04,0x35,0x3F,0xF2,0x04,0x35,0xBF,0xBB,0x1E,0x39,0x3D,0x21,0x00,0x00,0x00,
	0x26,0x00,0x00,0x00,0x1F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xDF,0x6B,0x8D,0x27,
	0xF4,0x04,0x35,0x3F,0xF2,0x04,0x35,0xBF,0xBB,0x1E,0x39,0x3D,0x29,0x00,0x00,0x00,
	0x27,0x00,0x00,0x00,0x28,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBE,0xFE,0x3D,0x3F,0x28,0x00,0x00,0x00,
	0x27,0x00,0x00,0x00,0x2B,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,
	0x00,0x00,0x00,0x80,0x00,0x00,0x00,0x00,0xBE,0xFE,0x3D,0x3F,0x2B,0x00,0x00,0x00,
	0x27,0x00,0x00,0x00,0x2A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xBE,0xFE,0x3D,0x3F,0x2A,0x00,0x00,0x00,
	0x27,0x00,0x00,0x00,0x29,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xBE,0xFE,0x3D,0x3F,0x2E,0x00,0x00,0x00,
	0x2C,0x00,0x00,0x00,0x2D,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xBE,0xFE,0x3D,0xBF,0x2D,0x00,0x00,0x00,
	0x2C,0x00,0x00,0x00,0x2F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,
	0x00,0x00,0x00,0x80,0x00,0x00,0x00,0x00,0xBE,0xFE,0x3D,0xBF,0x2F,0x00,0x00,0x00,
	0x2C,0x00,0x00,0x00,0x30,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBE,0xFE,0x3D,0xBF,0x30,0x00,0x00,0x00,
	0x2C,0x00,0x00,0x00,0x2E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xBE,0xFE,0x3D,0xBF,0x3B,0x00,0x00,0x00,
	0x39,0x00,0x00,0x00,0x3A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3A,0x00,0x00,0x00,
	0x39,0x00,0x00,0x00,0x3D,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3D,0x00,0x00,0x00,
	0x39,0x00,0x00,0x00,0x3C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3C,0x00,0x00,0x00,
	0x39,0x00,0x00,0x00,0x3B,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x40,0x00,0x00,0x00,
	0x3E,0x00,0x00,0x00,0x3F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFD,0xFF,0xFF,0xB2,
	0xF4,0x04,0xB5,0xB3,0x00,0x00,0x80,0xBF,0x12,0x99,0xD4,0xAE,0x3F,0x00,0x00,0x00,
	0x3E,0x00,0x00,0x00,0x41,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF,0xB2,
	0xF3,0x04,0xB5,0xB3,0x00,0x00,0x80,0xBF,0x0A,0x99,0xD4,0xAE,0x41,0x00,0x00,0x00,
	0x3E,0x00,0x00,0x00,0x42,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF,0xB2,
	0xF4,0x04,0xB5,0xB3,0x00,0x00,0x80,0xBF,0x0E,0x99,0xD4,0xAE,0x42,0x00,0x00,0x00,
	0x3E,0x00,0x00,0x00,0x40,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFD,0xFF,0xFF,0xB2,
	0xF4,0x04,0xB5,0xB3,0x00,0x00,0x80,0xBF,0x12,0x99,0xD4,0xAE,0x45,0x00,0x00,0x00,
	0x43,0x00,0x00,0x00,0x44,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF3,0x04,0x35,0xBF,
	0xF5,0x04,0x35,0xBF,0x15,0x7A,0xAD,0x33,0xBB,0x1E,0x39,0x3D,0x44,0x00,0x00,0x00,
	0x43,0x00,0x00,0x00,0x46,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF3,0x04,0x35,0xBF,
	0xF5,0x04,0x35,0xBF,0x15,0x7A,0xAD,0x33,0xBB,0x1E,0x39,0x3D,0x44,0x00,0x00,0x00,
	0x46,0x00,0x00,0x00,0x47,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF5,0x04,0x35,0x3F,
	0xF2,0x04,0x35,0xBF,0x31,0xEF,0x25,0x33,0x0F,0xC1,0x0B,0x3D,0x47,0x00,0x00,0x00,
	0x46,0x00,0x00,0x00,0x48,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF5,0x04,0x35,0x3F,
	0xF2,0x04,0x35,0xBF,0x31,0xEF,0x25,0x33,0x0F,0xC1,0x0B,0x3D,0x47,0x00,0x00,0x00,
	0x48,0x00,0x00,0x00,0x49,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF2,0x04,0x35,0x3F,
	0xF5,0x04,0x35,0x3F,0x15,0x7A,0xAD,0xB3,0xE5,0x6F,0x22,0x3D,0x49,0x00,0x00,0x00,
	0x48,0x00,0x00,0x00,0x4A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF2,0x04,0x35,0x3F,
	0xF5,0x04,0x35,0x3F,0xF4,0x04,0xB5,0xB3,0xE5,0x6F,0x22,0x3D,0x49,0x00,0x00,0x00,
	0x4A,0x00,0x00,0x00,0x45,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF4,0x04,0x35,0xBF,
	0xF2,0x04,0x35,0x3F,0xF1,0x04,0x35,0xB3,0x90,0xCD,0x4F,0x3D,0x45,0x00,0x00,0x00,
	0x4A,0x00,0x00,0x00,0x43,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF4,0x04,0x35,0xBF,
	0xF3,0x04,0x35,0x3F,0x33,0xEF,0x25,0xB3,0x91,0xCD,0x4F,0x3D,0x4D,0x00,0x00,0x00,
	0x4B,0x00,0x00,0x00,0x4C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,0x00,0x00,0x40,0x3F,0x4C,0x00,0x00,0x00,
	0x4B,0x00,0x00,0x00,0x4F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,0x00,0x00,0x40,0x3F,0x4F,0x00,0x00,0x00,
	0x4B,0x00,0x00,0x00,0x4E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x80,0x00,0x00,0x80,0x3F,0x00,0x00,0x40,0x3F,0x4E,0x00,0x00,0x00,
	0x4B,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,
	0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,0x00,0x00,0x40,0x3F,0x52,0x00,0x00,0x00,
	0x50,0x00,0x00,0x00,0x51,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x80,0x00,0x00,0x80,0xBF,0x00,0x00,0x40,0xBF,0x51,0x00,0x00,0x00,
	0x50,0x00,0x00,0x00,0x53,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,0x00,0x00,0x40,0xBF,0x53,0x00,0x00,0x00,
	0x50,0x00,0x00,0x00,0x54,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,0x00,0x00,0x40,0xBF,0x54,0x00,0x00,0x00,
	0x50,0x00,0x00,0x00,0x52,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,
	0x00,0x00,0x00,0x00,0x00,0x00,0x80,0xBF,0x00,0x00,0x40,0xBF,0x57,0x00,0x00,0x00,
	0x55,0x00,0x00,0x00,0x56,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x56,0x00,0x00,0x00,
	0x55,0x00,0x00,0x00,0x58,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF2,0xEA,0x24,0xBF,
	0xF4,0xEA,0x24,0xBF,0x46,0x18,0xD3,0x3E,0x8D,0xAD,0xD5,0x3E,0x56,0x00,0x00,0x00,
	0x58,0x00,0x00,0x00,0x59,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x59,0x00,0x00,0x00,
	0x58,0x00,0x00,0x00,0x5A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF6,0xEA,0x24,0x3F,
	0xF2,0xEA,0x24,0xBF,0x42,0x18,0xD3,0x3E,0x62,0x58,0xCB,0x3E,0x59,0x00,0x00,0x00,
	0x5A,0x00,0x00,0x00,0x5B,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5B,0x00,0x00,0x00,
	0x5A,0x00,0x00,0x00,0x5C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF4,0xEA,0x24,0x3F,
	0xF6,0xEA,0x24,0x3F,0x3F,0x18,0xD3,0x3E,0xF7,0x82,0xD0,0x3E,0x5B,0x00,0x00,0x00,
	0x5C,0x00,0x00,0x00,0x57,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x57,0x00,0x00,0x00,
	0x5C,0x00,0x00,0x00,0x55,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF4,0xEA,0x24,0xBF,
	0xF4,0xEA,0x24,0x3F,0x42,0x18,0xD3,0x3E,0x22,0xD8,0xDA,0x3E,0x5F,0x00,0x00,0x00,
	0x5D,0x00,0x00,0x00,0x5E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x5E,0x00,0x00,0x00,
	0x5D,0x00,0x00,0x00,0x61,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x61,0x00,0x00,0x00,
	0x5D,0x00,0x00,0x00,0x60,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x60,0x00,0x00,0x00,
	0x5D,0x00,0x00,0x00,0x5F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x64,0x00,0x00,0x00,
	0x62,0x00,0x00,0x00,0x63,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x55,0x55,0x05,0xB3,
	0x00,0x00,0x80,0xBF,0x56,0x55,0xB7,0x33,0x69,0x50,0x80,0xBB,0x63,0x00,0x00,0x00,
	0x62,0x00,0x00,0x00,0x65,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x53,0x55,0x05,0xB3,
	0x00,0x00,0x80,0xBF,0x56,0x55,0xB7,0x33,0x69,0x50,0x80,0xBB,0x65,0x00,0x00,0x00,
	0x62,0x00,0x00,0x00,0x66,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x53,0x55,0x05,0xB3,
	0x00,0x00,0x80,0xBF,0x56,0x55,0xB7,0x33,0x69,0x50,0x80,0xBB,0x66,0x00,0x00,0x00,
	0x62,0x00,0x00,0x00,0x64,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x55,0x55,0x05,0xB3,
	0x00,0x00,0x80,0xBF,0x56,0x55,0xB7,0x33,0x69,0x50,0x80,0xBB,0x69,0x00,0x00,0x00,
	0x67,0x00,0x00,0x00,0x68,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF3,0x04,0x35,0xBF,
	0xF4,0x04,0xB5,0x32,0xF3,0x04,0x35,0x3F,0x24,0x76,0x44,0x3D,0x68,0x00,0x00,0x00,
	0x67,0x00,0x00,0x00,0x6A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF3,0x04,0x35,0xBF,
	0xF3,0x04,0xB5,0x32,0xF3,0x04,0x35,0x3F,0x24,0x76,0x44,0x3D,0x68,0x00,0x00,0x00,
	0x6A,0x00,0x00,0x00,0x6B,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF5,0x04,0x35,0x3F,
	0xF5,0x04,0xB5,0xB2,0xF2,0x04,0x35,0x3F,0x7A,0x18,0x17,0x3D,0x6B,0x00,0x00,0x00,
	0x6A,0x00,0x00,0x00,0x6C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF5,0x04,0x35,0x3F,
	0xF5,0x04,0xB5,0xB2,0xF2,0x04,0x35,0x3F,0x7A,0x18,0x17,0x3D,0x6B,0x00,0x00,0x00,
	0x6C,0x00,0x00,0x00,0x6D,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF2,0x04,0x35,0x3F,
	0xF2,0x04,0xB5,0xB2,0xF5,0x04,0x35,0xBF,0x7A,0x18,0x17,0x3D,0x6D,0x00,0x00,0x00,
	0x6C,0x00,0x00,0x00,0x6E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF2,0x04,0x35,0x3F,
	0xF2,0x04,0xB5,0xB2,0xF5,0x04,0x35,0xBF,0x7A,0x18,0x17,0x3D,0x6D,0x00,0x00,0x00,
	0x6E,0x00,0x00,0x00,0x69,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF5,0x04,0x35,0xBF,
	0xF5,0x04,0xB5,0x32,0xF2,0x04,0x35,0xBF,0x26,0x76,0x44,0x3D,0x69,0x00,0x00,0x00,
	0x6E,0x00,0x00,0x00,0x67,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF5,0x04,0x35,0xBF,
	0xF5,0x04,0xB5,0x32,0xF2,0x04,0x35,0xBF,0x26,0x76,0x44,0x3D,0x71,0x00,0x00,0x00,
	0x6F,0x00,0x00,0x00,0x70,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x80,0x3F,0x00,0x00,0x00,0x80,0xA1,0x00,0x41,0x3F,0x70,0x00,0x00,0x00,
	0x6F,0x00,0x00,0x00,0x73,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x80,0x3F,0x00,0x00,0x00,0x00,0xA1,0x00,0x41,0x3F,0x73,0x00,0x00,0x00,
	0x6F,0x00,0x00,0x00,0x72,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x80,0x3F,0x00,0x00,0x00,0x00,0xA1,0x00,0x41,0x3F,0x72,0x00,0x00,0x00,
	0x6F,0x00,0x00,0x00,0x71,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,
	0x00,0x00,0x80,0x3F,0x00,0x00,0x00,0x00,0xA1,0x00,0x41,0x3F,0x76,0x00,0x00,0x00,
	0x74,0x00,0x00,0x00,0x75,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x80,0xBF,0x00,0x00,0x00,0x00,0xA1,0x00,0x41,0xBF,0x75,0x00,0x00,0x00,
	0x74,0x00,0x00,0x00,0x77,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x80,0xBF,0x00,0x00,0x00,0x00,0xA1,0x00,0x41,0xBF,0x77,0x00,0x00,0x00,
	0x74,0x00,0x00,0x00,0x78,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x80,0xBF,0x00,0x00,0x00,0x80,0xA1,0x00,0x41,0xBF,0x78,0x00,0x00,0x00,
	0x74,0x00,0x00,0x00,0x76,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,
	0x00,0x00,0x80,0xBF,0x00,0x00,0x00,0x00,0xA1,0x00,0x41,0xBF,0x7B,0x00,0x00,0x00,
	0x79,0x00,0x00,0x00,0x7A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7A,0x00,0x00,0x00,
	0x79,0x00,0x00,0x00,0x7C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF3,0xEA,0x24,0xBF,
	0x46,0x18,0xD3,0x3E,0xF4,0xEA,0x24,0x3F,0xD1,0xCB,0xD7,0x3E,0x7A,0x00,0x00,0x00,
	0x7C,0x00,0x00,0x00,0x7D,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7D,0x00,0x00,0x00,
	0x7C,0x00,0x00,0x00,0x7E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF6,0xEA,0x24,0x3F,
	0x43,0x18,0xD3,0x3E,0xF2,0xEA,0x24,0x3F,0xA6,0x76,0xCD,0x3E,0x7D,0x00,0x00,0x00,
	0x7E,0x00,0x00,0x00,0x7F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7F,0x00,0x00,0x00,
	0x7E,0x00,0x00,0x00,0x80,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF3,0xEA,0x24,0x3F,
	0x43,0x18,0xD3,0x3E,0xF5,0xEA,0x24,0xBF,0xF1,0x0B,0xD0,0x3E,0x7F,0x00,0x00,0x00,
	0x80,0x00,0x00,0x00,0x7B,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7B,0x00,0x00,0x00,
	0x80,0x00,0x00,0x00,0x79,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xF4,0xEA,0x24,0xBF,
	0x46,0x18,0xD3,0x3E,0xF2,0xEA,0x24,0xBF,0x1B,0x61,0xDA,0x3E,0x83,0x00,0x00,0x00,
	0x81,0x00,0x00,0x00,0x82,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x82,0x00,0x00,0x00,
	0x81,0x00,0x00,0x00,0x85,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x85,0x00,0x00,0x00,
	0x81,0x00,0x00,0x00,0x84,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x84,0x00,0x00,0x00,
	0x81,0x00,0x00,0x00,0x83,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x8F,0x84,0xC1,0xBE,
	0xBD,0x50,0x20,0x3E,0xBF,0x98,0x69,0x3F,0xCA,0xDA,0x44,0x3E,0x02,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC1,0x50,0x20,0xBE,
	0x8F,0x84,0xC1,0x3E,0xBF,0x98,0x69,0x3F,0x83,0xF7,0x43,0x3E,0x02,0x00,0x00,0x00,
	0x01,0x00,0x00,0x00,0x0A,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x05,0xFE,0xB6,0xBE,
	0x26,0xE4,0x5C,0x3F,0x01,0xFE,0xB6,0x3E,0x8C,0x03,0x3D,0x3E,0x0A,0x00,0x00,0x00,
	0x01,0x00,0x00,0x00,0x09,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0xFE,0xB6,0xBE,
	0x25,0xE4,0x5C,0x3F,0x02,0xFE,0xB6,0x3E,0x8C,0x03,0x3D,0x3E,0x0A,0x00,0x00,0x00,
	0x09,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0xFE,0xB6,0xBE,
	0x24,0xE4,0x5C,0x3F,0x05,0xFE,0xB6,0xBE,0x8C,0x03,0x3D,0x3E,0x12,0x00,0x00,0x00,
	0x09,0x00,0x00,0x00,0x11,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x03,0xFE,0xB6,0xBE,
	0x24,0xE4,0x5C,0x3F,0x07,0xFE,0xB6,0xBE,0x8C,0x03,0x3D,0x3E,0x12,0x00,0x00,0x00,
	0x11,0x00,0x00,0x00,0x19,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xBD,0x50,0x20,0xBE,
	0x8C,0x84,0xC1,0x3E,0xBF,0x98,0x69,0xBF,0x83,0xF7,0x43,0x3E,0x19,0x00,0x00,0x00,
	0x11,0x00,0x00,0x00,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xBA,0x50,0x20,0x3E,
	0x8D,0x84,0xC1,0x3E,0xBF,0x98,0x69,0xBF,0xAD,0x74,0x41,0x3E,0x19,0x00,0x00,0x00,
	0x18,0x00,0x00,0x00,0x17,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x8D,0x84,0xC1,0x3E,
	0xC2,0x50,0x20,0x3E,0xC0,0x98,0x69,0xBF,0xDA,0xCA,0x3E,0x3E,0x17,0x00,0x00,0x00,
	0x18,0x00,0x00,0x00,0x0F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x24,0xE4,0x5C,0x3F,
	0x07,0xFE,0xB6,0x3E,0x06,0xFE,0xB6,0xBE,0x77,0x33,0x31,0x3E,0x17,0x00,0x00,0x00,
	0x0F,0x00,0x00,0x00,0x0E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x26,0xE4,0x5C,0x3F,
	0x00,0xFE,0xB6,0xBE,0x07,0xFE,0xB6,0xBE,0xB3,0x55,0x2E,0x3E,0x0E,0x00,0x00,0x00,
	0x0F,0x00,0x00,0x00,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x27,0xE4,0x5C,0x3F,
	0x02,0xFE,0xB6,0xBE,0x02,0xFE,0xB6,0x3E,0xB4,0x55,0x2E,0x3E,0x0E,0x00,0x00,0x00,
	0x06,0x00,0x00,0x00,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0A,0xFE,0xB6,0x3E,
	0x24,0xE4,0x5C,0xBF,0x01,0xFE,0xB6,0x3E,0x8C,0x5C,0x30,0x3E,0x05,0x00,0x00,0x00,
	0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC3,0x50,0x20,0x3E,
	0x8F,0x84,0xC1,0xBE,0xBF,0x98,0x69,0x3F,0xB5,0x6C,0x3E,0x3E,0x05,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xBA,0x50,0x20,0xBE,
	0x8F,0x84,0xC1,0xBE,0xBE,0x98,0x69,0x3F,0x8A,0xEF,0x40,0x3E,0x04,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x03,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x8E,0x84,0xC1,0xBE,
	0xBF,0x50,0x20,0xBE,0xBE,0x98,0x69,0x3F,0x5E,0x99,0x43,0x3E,0x04,0x00,0x00,0x00,
	0x03,0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x24,0xE4,0x5C,0xBF,
	0x07,0xFE,0xB6,0xBE,0x04,0xFE,0xB6,0x3E,0xA2,0x2C,0x3C,0x3E,0x0C,0x00,0x00,0x00,
	0x03,0x00,0x00,0x00,0x0B,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x24,0xE4,0x5C,0xBF,
	0x08,0xFE,0xB6,0xBE,0x03,0xFE,0xB6,0x3E,0xA2,0x2C,0x3C,0x3E,0x0C,0x00,0x00,0x00,
	0x0B,0x00,0x00,0x00,0x14,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x23,0xE4,0x5C,0xBF,
	0x06,0xFE,0xB6,0xBE,0x08,0xFE,0xB6,0xBE,0xA1,0x2C,0x3C,0x3E,0x14,0x00,0x00,0x00,
	0x0B,0x00,0x00,0x00,0x13,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x24,0xE4,0x5C,0xBF,
	0x0A,0xFE,0xB6,0xBE,0x07,0xFE,0xB6,0xBE,0xA2,0x2C,0x3C,0x3E,0x14,0x00,0x00,0x00,
	0x13,0x00,0x00,0x00,0x19,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x8E,0x84,0xC1,0xBE,
	0xC0,0x50,0x20,0xBE,0xC0,0x98,0x69,0xBF,0x60,0x99,0x43,0x3E,0x19,0x00,0x00,0x00,
	0x13,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x8F,0x84,0xC1,0xBE,
	0xBB,0x50,0x20,0x3E,0xC0,0x98,0x69,0xBF,0xCB,0xDA,0x44,0x3E,0x0B,0x00,0x00,0x00,
	0x0A,0x00,0x00,0x00,0x13,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x25,0xE4,0x5C,0xBF,
	0x01,0xFE,0xB6,0x3E,0x08,0xFE,0xB6,0xBE,0x66,0x0A,0x3F,0x3E,0x13,0x00,0x00,0x00,
	0x0A,0x00,0x00,0x00,0x12,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x25,0xE4,0x5C,0xBF,
	0x01,0xFE,0xB6,0x3E,0x08,0xFE,0xB6,0xBE,0x66,0x0A,0x3F,0x3E,0x0A,0x00,0x00,0x00,
	0x0B,0x00,0x00,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x26,0xE4,0x5C,0xBF,
	0x02,0xFE,0xB6,0x3E,0x04,0xFE,0xB6,0x3E,0x66,0x0A,0x3F,0x3E,0x02,0x00,0x00,0x00,
	0x0B,0x00,0x00,0x00,0x03,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x25,0xE4,0x5C,0xBF,
	0x04,0xFE,0xB6,0x3E,0x02,0xFE,0xB6,0x3E,0x66,0x0A,0x3F,0x3E,0x0C,0x00,0x00,0x00,
	0x14,0x00,0x00,0x00,0x15,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0xFE,0xB6,0xBE,
	0x24,0xE4,0x5C,0xBF,0x09,0xFE,0xB6,0xBE,0x15,0x18,0x36,0x3E,0x15,0x00,0x00,0x00,
	0x14,0x00,0x00,0x00,0x19,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xBB,0x50,0x20,0xBE,
	0x8F,0x84,0xC1,0xBE,0xC0,0x98,0x69,0xBF,0x8B,0xEF,0x40,0x3E,0x15,0x00,0x00,0x00,
	0x19,0x00,0x00,0x00,0x16,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xC0,0x50,0x20,0x3E,
	0x8E,0x84,0xC1,0xBE,0xC0,0x98,0x69,0xBF,0xB6,0x6C,0x3E,0x3E,0x16,0x00,0x00,0x00,
	0x19,0x00,0x00,0x00,0x17,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x8E,0x84,0xC1,0x3E,
	0xB7,0x50,0x20,0xBE,0xBF,0x98,0x69,0xBF,0x6E,0x89,0x3D,0x3E,0x16,0x00,0x00,0x00,
	0x17,0x00,0x00,0x00,0x0E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x27,0xE4,0x5C,0x3F,
	0xFE,0xFD,0xB6,0xBE,0x04,0xFE,0xB6,0xBE,0xB3,0x55,0x2E,0x3E,0x16,0x00,0x00,0x00,
	0x0D,0x00,0x00,0x00,0x15,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x08,0xFE,0xB6,0x3E,
	0x23,0xE4,0x5C,0xBF,0x06,0xFE,0xB6,0xBE,0x8D,0x5C,0x30,0x3E,0x15,0x00,0x00,0x00,
	0x0D,0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFE,0xB6,0xBE,
	0x26,0xE4,0x5C,0xBF,0x08,0xFE,0xB6,0xBE,0x16,0x18,0x36,0x3E,0x16,0x00,0x00,0x00,
	0x0E,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0A,0xFE,0xB6,0x3E,
	0x24,0xE4,0x5C,0xBF,0x05,0xFE,0xB6,0xBE,0x8E,0x5C,0x30,0x3E,0x0D,0x00,0x00,0x00,
	0x0E,0x00,0x00,0x00,0x05,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x0B,0xFE,0xB6,0x3E,
	0x24,0xE4,0x5C,0xBF,0x03,0xFE,0xB6,0x3E,0x8D,0x5C,0x30,0x3E,0x0D,0x00,0x00,0x00,
	0x05,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0xFE,0xB6,0xBE,
	0x26,0xE4,0x5C,0xBF,0x03,0xFE,0xB6,0x3E,0x15,0x18,0x36,0x3E,0x04,0x00,0x00,0x00,
	0x0C,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0xFE,0xB6,0xBE,
	0x26,0xE4,0x5C,0xBF,0x05,0xFE,0xB6,0x3E,0x15,0x18,0x36,0x3E,0x06,0x00,0x00,0x00,
	0x0F,0x00,0x00,0x00,0x07,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x26,0xE4,0x5C,0x3F,
	0xFF,0xFD,0xB6,0xBE,0x02,0xFE,0xB6,0x3E,0xB3,0x55,0x2E,0x3E,0x07,0x00,0x00,0x00,
	0x0F,0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x24,0xE4,0x5C,0x3F,
	0x0A,0xFE,0xB6,0x3E,0x03,0xFE,0xB6,0x3E,0x77,0x33,0x31,0x3E,0x07,0x00,0x00,0x00,
	0x10,0x00,0x00,0x00,0x08,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x26,0xE4,0x5C,0x3F,
	0x09,0xFE,0xB6,0x3E,0x01,0xFE,0xB6,0x3E,0x77,0x33,0x31,0x3E,0x08,0x00,0x00,0x00,
	0x10,0x00,0x00,0x00,0x09,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x02,0xFE,0xB6,0x3E,
	0x26,0xE4,0x5C,0x3F,0x02,0xFE,0xB6,0x3E,0x05,0x48,0x37,0x3E,0x08,0x00,0x00,0x00,
	0x09,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFD,0xB6,0x3E,
	0x26,0xE4,0x5C,0x3F,0x03,0xFE,0xB6,0x3E,0x05,0x48,0x37,0x3E,0x08,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x07,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x8D,0x84,0xC1,0x3E,
	0xC2,0x50,0x20,0x3E,0xBE,0x98,0x69,0x3F,0xD8,0xCA,0x3E,0x3E,0x07,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x06,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x90,0x84,0xC1,0x3E,
	0xBB,0x50,0x20,0xBE,0xBF,0x98,0x69,0x3F,0x6E,0x89,0x3D,0x3E,0x0F,0x00,0x00,0x00,
	0x18,0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x24,0xE4,0x5C,0x3F,
	0x0A,0xFE,0xB6,0x3E,0x05,0xFE,0xB6,0xBE,0x78,0x33,0x31,0x3E,0x10,0x00,0x00,0x00,
	0x18,0x00,0x00,0x00,0x11,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xFF,0xFD,0xB6,0x3E,
	0x25,0xE4,0x5C,0x3F,0x05,0xFE,0xB6,0xBE,0x04,0x48,0x37,0x3E,0x10,0x00,0x00,0x00,
	0x11,0x00,0x00,0x00,0x09,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0xFE,0xB6,0x3E,
	0x25,0xE4,0x5C,0x3F,0x08,0xFE,0xB6,0xBE,0x05,0x48,0x37,0x3E,0x00,0x00,0x00,0x00,
	0x08,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0xBA,0x50,0x20,0x3E,
	0x8F,0x84,0xC1,0x3E,0xBE,0x98,0x69,0x3F,0xAC,0x74,0x41,0x3E,0x22,0x00,0x00,0x00,
	0x18,0x02,0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x02,0x00,0x00,0x00,
	0x03,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x05,0x00,0x00,0x00,0x06,0x00,0x00,0x00,
	0x07,0x00,0x00,0x00,0x08,0x00,0x00,0x00,0x09,0x00,0x00,0x00,0x0A,0x00,0x00,0x00,
	0x0B,0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0x0D,0x00,0x00,0x00,0x0E,0x00,0x00,0x00,
	0x0F,0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x11,0x00,0x00,0x00,0x12,0x00,0x00,0x00,
	0x13,0x00,0x00,0x00,0x14,0x00,0x00,0x00,0x15,0x00,0x00,0x00,0x16,0x00,0x00,0x00,
	0x17,0x00,0x00,0x00,0x18,0x00,0x00,0x00,0x19,0x00,0x00,0x00,0x1A,0x00,0x00,0x00,
	0x1B,0x00,0x00,0x00,0x1C,0x00,0x00,0x00,0x1D,0x00,0x00,0x00,0x1E,0x00,0x00,0x00,
	0x1F,0x00,0x00,0x00,0x20,0x00,0x00,0x00,0x21,0x00,0x00,0x00,0x22,0x00,0x00,0x00,
	0x23,0x00,0x00,0x00,0x24,0x00,0x00,0x00,0x25,0x00,0x00,0x00,0x26,0x00,0x00,0x00,
	0x27,0x00,0x00,0x00,0x28,0x00,0x00,0x00,0x29,0x00,0x00,0x00,0x2A,0x00,0x00,0x00,
	0x2B,0x00,0x00,0x00,0x2C,0x00,0x00,0x00,0x2D,0x00,0x00,0x00,0x2E,0x00,0x00,0x00,
	0x2F,0x00,0x00,0x00,0x30,0x00,0x00,0x00,0x31,0x00,0x00,0x00,0x32,0x00,0x00,0x00,
	0x33,0x00,0x00,0x00,0x34,0x00,0x00,0x00,0x35,0x00,0x00,0x00,0x36,0x00,0x00,0x00,
	0x37,0x00,0x00,0x00,0x38,0x00,0x00,0x00,0x39,0x00,0x00,0x00,0x3A,0x00,0x00,0x00,
	0x3B,0x00,0x00,0x00,0x3C,0x00,0x00,0x00,0x3D,0x00,0x00,0x00,0x3E,0x00,0x00,0x00,
	0x3F,0x00,0x00,0x00,0x40,0x00,0x00,0x00,0x41,0x00,0x00,0x00,0x42,0x00,0x00,0x00,
	0x43,0x00,0x00,0x00,0x44,0x00,0x00,0x00,0x45,0x00,0x00,0x00,0x46,0x00,0x00,0x00,
	0x47,0x00,0x00,0x00,0x48,0x00,0x00,0x00,0x49,0x00,0x00,0x00,0x4A,0x00,0x00,0x00,
	0x4B,0x00,0x00,0x00,0x4C,0x00,0x00,0x00,0x4D,0x00,0x00,0x00,0x4E,0x00,0x00,0x00,
	0x4F,0x00,0x00,0x00,0x50,0x00,0x00,0x00,0x51,0x00,0x00,0x00,0x52,0x00,0x00,0x00,
	0x53,0x00,0x00,0x00,0x54,0x00,0x00,0x00,0x55,0x00,0x00,0x00,0x56,0x00,0x00,0x00,
	0x57,0x00,0x00,0x00,0x58,0x00,0x00,0x00,0x59,0x00,0x00,0x00,0x5A,0x00,0x00,0x00,
	0x5B,0x00,0x00,0x00,0x5C,0x00,0x00,0x00,0x5D,0x00,0x00,0x00,0x5E,0x00,0x00,0x00,
	0x5F,0x00,0x00,0x00,0x60,0x00,0x00,0x00,0x61,0x00,0x00,0x00,0x62,0x00,0x00,0x00,
	0x63,0x00,0x00,0x00,0x64,0x00,0x00,0x00,0x65,0x00,0x00,0x00,0x66,0x00,0x00,0x00,
	0x67,0x00,0x00,0x00,0x68,0x00,0x00,0x00,0x69,0x00,0x00,0x00,0x6A,0x00,0x00,0x00,
	0x6B,0x00,0x00,0x00,0x6C,0x00,0x00,0x00,0x6D,0x00,0x00,0x00,0x6E,0x00,0x00,0x00,
	0x6F,0x00,0x00,0x00,0x70,0x00,0x00,0x00,0x71,0x00,0x00,0x00,0x72,0x00,0x00,0x00,
	0x73,0x00,0x00,0x00,0x74,0x00,0x00,0x00,0x75,0x00,0x00,0x00,0x76,0x00,0x00,0x00,
	0x77,0x00,0x00,0x00,0x78,0x00,0x00,0x00,0x79,0x00,0x00,0x00,0x7A,0x00,0x00,0x00,
	0x7B,0x00,0x00,0x00,0x7C,0x00,0x00,0x00,0x7D,0x00,0x00,0x00,0x7E,0x00,0x00,0x00,
	0x7F,0x00,0x00,0x00,0x80,0x00,0x00,0x00,0x81,0x00,0x00,0x00,0x82,0x00,0x00,0x00,
	0x83,0x00,0x00,0x00,0x84,0x00,0x00,0x00,0x85,0x00,0x00,0x00,0x28,0x00,0x00,0x00,
	0x10,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x00,0x01,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x2A,0x00,0x00,0x00,0x44,0x00,0x00,0x00,0x2B,0x00,0x00,0x00,
	0x3C,0x00,0x00,0x00,0x2C,0x00,0x00,0x00,0x0C,0x00,0x00,0x00,0x4D,0x61,0x74,0x65,
	0x72,0x69,0x61,0x6C,0x20,0x23,0x31,0x00,0x2D,0x00,0x00,0x00,0x20,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0xFF,0xFF,0xFF,0x00,0xFF,0xFF,0xFF,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x80,0x3F,0x00,0x00,0x00,0x00,
	0x29,0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x03,0x01,0x00,0x00,0x00,0x01,0x00,0x01,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x38,0x00,0x00,0x00,0x38,0x02,0x00,0x00,
	0x39,0x00,0x00,0x00,0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3A,0x00,0x00,0x00,
	0x04,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3B,0x00,0x00,0x00,0x18,0x02,0x00,0x00,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFD,0x00,0x00,0xFF,
	0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,
	0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,
	0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,
	0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,
	0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,
	0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,
	0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,
	0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,
	0xFD,0x00,0x00,0xFF,0xFD,0x00,0x00,0xFF,0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,
	0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,
	0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,
	0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,
	0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,
	0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,
	0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,
	0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,
	0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,
	0x12,0x00,0xFC,0xFF,0x12,0x00,0xFC,0xFF,0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,
	0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,
	0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,
	0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,
	0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,
	0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,
	0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,
	0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,
	0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF,
	0x00,0xF5,0x1D,0xFF,0x00,0xF5,0x1D,0xFF
};
#endif